"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var observable_1 = require("tns-core-modules/data/observable");
var InternalEventManager = (function () {
    function InternalEventManager() {
        this.notificationCenter = observable_1.fromObject({});
        this.listenerCounter = 0;
        this.callbacks = new CallbackStore();
    }
    InternalEventManager.prototype.on = function (eventName, callback) {
        var callbackId = [eventName, this.listenerCounter];
        var internalCallback = function (eventData) {
            return callback(eventData.data);
        };
        this.callbacks.set(callbackId, internalCallback);
        this.notificationCenter.on(eventName, internalCallback);
        return this.listenerCounter++;
    };
    InternalEventManager.prototype.off = function (eventName, listenerId) {
        if (listenerId === undefined) {
            this.notificationCenter.off(eventName);
            this.callbacks.deleteCallbackMap(eventName);
            return;
        }
        var callbackId = [eventName, listenerId];
        var internalCallback = this.callbacks.get(callbackId);
        if (!internalCallback) {
            return;
        }
        this.notificationCenter.off(eventName, internalCallback);
        this.callbacks.delete(callbackId);
    };
    InternalEventManager.prototype.emit = function (dispatchableEvent) {
        var internalEventData = {
            eventName: dispatchableEvent.name,
            object: this.notificationCenter,
            data: __assign({}, dispatchableEvent),
        };
        try {
            this.notificationCenter.notify(internalEventData);
        }
        catch (err) {
            if (err instanceof TypeError) {
                return;
            }
            throw err;
        }
    };
    InternalEventManager.prototype.hasListeners = function (eventName) {
        return this.notificationCenter.hasListeners(eventName);
    };
    return InternalEventManager;
}());
exports.InternalEventManager = InternalEventManager;
var CallbackStore = (function () {
    function CallbackStore() {
        this.callbackTree = {};
    }
    CallbackStore.prototype.set = function (callbackId, internalCallback) {
        var _a = __read(callbackId, 2), eventName = _a[0], listenerId = _a[1];
        if (!this.callbackTree[eventName]) {
            this.callbackTree[eventName] = new Map();
        }
        this.callbackTree[eventName].set(listenerId, internalCallback);
    };
    CallbackStore.prototype.get = function (callbackId) {
        var _a = __read(callbackId, 2), eventName = _a[0], listenerId = _a[1];
        var callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return null;
        }
        var internalCallback = callbackMap.get(listenerId);
        return internalCallback ? internalCallback : null;
    };
    CallbackStore.prototype.delete = function (callbackId) {
        var _a = __read(callbackId, 2), eventName = _a[0], listenerId = _a[1];
        var callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return;
        }
        callbackMap.delete(listenerId);
        if (callbackMap.size === 0) {
            delete this.callbackTree[eventName];
        }
    };
    CallbackStore.prototype.deleteCallbackMap = function (eventName) {
        var callbackMap = this.callbackTree[eventName];
        if (!callbackMap) {
            return;
        }
        callbackMap.clear();
        delete this.callbackTree[eventName];
    };
    return CallbackStore;
}());
//# sourceMappingURL=internal-event-manager.js.map