import { PlannedTask, PlanningType } from "../tasks/planner/planned-task";
import { RunnableTask } from "../tasks/runnable-task";
export interface PlannedTasksStore {
    insert(plannedTask: PlannedTask): Promise<void>;
    delete(taskId: string): Promise<void>;
    get(task: RunnableTask | string): Promise<PlannedTask>;
    getAllSortedByNextRun(planningType: PlanningType): Promise<Array<PlannedTask>>;
    getAllCancelEvents(): Promise<Array<string>>;
    getAllFilteredByCancelEvent(cancelEvent: string): Promise<Array<PlannedTask>>;
    increaseErrorCount(taskId: string): Promise<void>;
    increaseTimeoutCount(taskId: string): Promise<void>;
    updateLastRun(taskId: string, timestamp: number): Promise<void>;
}
declare class PlannedTaskDBStore implements PlannedTasksStore {
    private readonly database;
    constructor();
    insert(plannedTask: PlannedTask): Promise<void>;
    delete(taskId: string): Promise<void>;
    get(task: string | RunnableTask): Promise<PlannedTask>;
    getAllSortedByNextRun(planningType?: PlanningType): Promise<Array<PlannedTask>>;
    getAllCancelEvents(): Promise<Array<string>>;
    getAllFilteredByCancelEvent(cancelEvent: string): Promise<Array<PlannedTask>>;
    increaseErrorCount(taskId: string): Promise<void>;
    increaseTimeoutCount(taskId: string): Promise<void>;
    updateLastRun(taskId: string, timestamp: number): Promise<void>;
    deleteAll(): Promise<void>;
}
export declare const plannedTasksDB: PlannedTaskDBStore;
export declare class PlannedTaskAlreadyExistsError extends Error {
    constructor({ name, startAt, interval, recurrent }: PlannedTask);
}
export {};
