"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var planned_task_1 = require("../tasks/planner/planned-task");
var nativescript_couchbase_plugin_1 = require("nativescript-couchbase-plugin");
var time_1 = require("../utils/time");
var DB_NAME = "task-dispatcher";
var DOC_TYPE = "planned-task";
var PlannedTaskDBStore = (function () {
    function PlannedTaskDBStore() {
        this.database = new nativescript_couchbase_plugin_1.Couchbase(DB_NAME);
    }
    PlannedTaskDBStore.prototype.insert = function (plannedTask) {
        return __awaiter(this, void 0, void 0, function () {
            var runnableTask, possibleTask, doc;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        runnableTask = runnableTaskFrom(plannedTask);
                        return [4, this.get(runnableTask)];
                    case 1:
                        possibleTask = _a.sent();
                        if (possibleTask) {
                            throw new PlannedTaskAlreadyExistsError(plannedTask);
                        }
                        doc = documentFrom(plannedTask);
                        this.database.createDocument(doc, plannedTask.id);
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.delete = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.database.deleteDocument(taskId);
                return [2];
            });
        });
    };
    PlannedTaskDBStore.prototype.get = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, _a, doc, name, startAt, interval, recurrent, docs, params, docs_1, docs_1_1, doc;
            return __generator(this, function (_b) {
                if (typeof task === "string") {
                    doc = this.database.getDocument(task);
                    return [2, doc ? plannedTaskFrom(doc) : null];
                }
                name = task.name, startAt = task.startAt, interval = task.interval, recurrent = task.recurrent;
                docs = this.database.query({
                    select: [],
                    where: [
                        { property: "name", comparison: "equalTo", value: name },
                        {
                            logical: nativescript_couchbase_plugin_1.QueryLogicalOperator.AND,
                            property: "startAt",
                            comparison: "equalTo",
                            value: startAt,
                        },
                        {
                            logical: nativescript_couchbase_plugin_1.QueryLogicalOperator.AND,
                            property: "interval",
                            comparison: "equalTo",
                            value: interval,
                        },
                        {
                            logical: nativescript_couchbase_plugin_1.QueryLogicalOperator.AND,
                            property: "recurrent",
                            comparison: "equalTo",
                            value: recurrent,
                        },
                    ],
                });
                if (docs.length === 0) {
                    return [2, null];
                }
                params = JSON.stringify(task.params);
                try {
                    for (docs_1 = __values(docs), docs_1_1 = docs_1.next(); !docs_1_1.done; docs_1_1 = docs_1.next()) {
                        doc = docs_1_1.value;
                        if (params === JSON.stringify(doc.params)) {
                            return [2, plannedTaskFrom(doc)];
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (docs_1_1 && !docs_1_1.done && (_a = docs_1.return)) _a.call(docs_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return [2, null];
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllSortedByNextRun = function (planningType) {
        return __awaiter(this, void 0, void 0, function () {
            var docs, plannedTasks, currentMillis;
            return __generator(this, function (_a) {
                docs = this.database.query({
                    select: [],
                    where: planningType
                        ? [
                            {
                                property: "planningType",
                                comparison: "equalTo",
                                value: planningType,
                            },
                        ]
                        : undefined,
                });
                plannedTasks = docs.map(function (doc) { return plannedTaskFrom(doc); });
                currentMillis = time_1.now();
                return [2, plannedTasks.sort(function (t1, t2) { return t1.nextRun(currentMillis) - t2.nextRun(currentMillis); })];
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllCancelEvents = function () {
        return __awaiter(this, void 0, void 0, function () {
            var docs, cancelEvents, cancelEventSet;
            return __generator(this, function (_a) {
                docs = this.database.query({ select: ["cancelEvent"] });
                cancelEvents = docs.map(function (doc) { return doc.cancelEvent; });
                cancelEventSet = new Set(cancelEvents);
                return [2, __spread(cancelEventSet)];
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllFilteredByCancelEvent = function (cancelEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var docs;
            return __generator(this, function (_a) {
                docs = this.database.query({
                    select: [],
                    where: [
                        { property: "cancelEvent", comparison: "equalTo", value: cancelEvent },
                    ],
                });
                return [2, docs.map(function (doc) { return plannedTaskFrom(doc); })];
            });
        });
    };
    PlannedTaskDBStore.prototype.increaseErrorCount = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (plannedTask) {
                            this.database.updateDocument(taskId, {
                                errorCount: plannedTask.errorCount + 1,
                            });
                        }
                        else {
                            throw new Error("Task not found: " + taskId);
                        }
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.increaseTimeoutCount = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (plannedTask) {
                            this.database.updateDocument(taskId, {
                                timeoutCount: plannedTask.timeoutCount + 1,
                            });
                        }
                        else {
                            throw new Error("Task not found: " + taskId);
                        }
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.updateLastRun = function (taskId, timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (plannedTask) {
                            this.database.updateDocument(taskId, {
                                lastRun: timestamp,
                            });
                        }
                        else {
                            throw new Error("Task not found: " + taskId);
                        }
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.deleteAll = function () {
        var _this = this;
        return new Promise(function (resolve) {
            var e_2, _a;
            var docs = _this.database.query({ select: [nativescript_couchbase_plugin_1.QueryMeta.ID] });
            try {
                for (var docs_2 = __values(docs), docs_2_1 = docs_2.next(); !docs_2_1.done; docs_2_1 = docs_2.next()) {
                    var doc = docs_2_1.value;
                    _this.database.deleteDocument(doc.id);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (docs_2_1 && !docs_2_1.done && (_a = docs_2.return)) _a.call(docs_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
            resolve();
        });
    };
    return PlannedTaskDBStore;
}());
function runnableTaskFrom(plannedTask) {
    var name = plannedTask.name, startAt = plannedTask.startAt, interval = plannedTask.interval, recurrent = plannedTask.recurrent, params = plannedTask.params, cancelEvent = plannedTask.cancelEvent;
    return {
        name: name,
        startAt: startAt,
        interval: interval,
        recurrent: recurrent,
        params: params,
        cancelEvent: cancelEvent,
    };
}
function documentFrom(plannedTask) {
    var runnableTask = runnableTaskFrom(plannedTask);
    var planningType = plannedTask.planningType, schedulerType = plannedTask.schedulerType, createdAt = plannedTask.createdAt, lastRun = plannedTask.lastRun, errorCount = plannedTask.errorCount, timeoutCount = plannedTask.timeoutCount;
    return __assign({ docType: DOC_TYPE }, runnableTask, { planningType: planningType,
        schedulerType: schedulerType,
        createdAt: createdAt,
        lastRun: lastRun,
        errorCount: errorCount,
        timeoutCount: timeoutCount });
}
function plannedTaskFrom(doc) {
    return new planned_task_1.PlannedTask(doc.planningType, doc.schedulerType, {
        name: doc.name,
        startAt: doc.startAt,
        interval: doc.interval,
        recurrent: doc.recurrent,
        params: doc.params,
        cancelEvent: doc.cancelEvent,
    }, doc.id, doc.createdAt, doc.lastRun, doc.errorCount, doc.timeoutCount);
}
exports.plannedTasksDB = new PlannedTaskDBStore();
var PlannedTaskAlreadyExistsError = (function (_super) {
    __extends(PlannedTaskAlreadyExistsError, _super);
    function PlannedTaskAlreadyExistsError(_a) {
        var name = _a.name, startAt = _a.startAt, interval = _a.interval, recurrent = _a.recurrent;
        return _super.call(this, "Already stored: {name=" + name + ", startAt=" + startAt + ", interval=" + interval + ", recurrent=" + recurrent + "}") || this;
    }
    return PlannedTaskAlreadyExistsError;
}(Error));
exports.PlannedTaskAlreadyExistsError = PlannedTaskAlreadyExistsError;
//# sourceMappingURL=planned-tasks-store.js.map