"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var planned_tasks_store_1 = require("../persistence/planned-tasks-store");
var time_based_1 = require("./schedulers/time-based");
var events_1 = require("../events");
var planned_task_1 = require("./planner/planned-task");
var logger_1 = require("../utils/logger");
var task_chain_1 = require("./task-chain");
var TaskCancelManager = (function () {
    function TaskCancelManager(taskStore, taskScheduler) {
        if (taskStore === void 0) { taskStore = planned_tasks_store_1.plannedTasksDB; }
        this.taskStore = taskStore;
        this.taskScheduler = taskScheduler;
        this.cancelEvents = new Set();
        this.logger = logger_1.getLogger("TaskCancelManager");
    }
    TaskCancelManager.prototype.init = function () {
        return __awaiter(this, void 0, void 0, function () {
            var cancelEvents;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskStore.getAllCancelEvents()];
                    case 1:
                        cancelEvents = _a.sent();
                        cancelEvents.forEach(function (cancelEvent) {
                            _this.listenToCancelEvent(cancelEvent);
                        });
                        return [2];
                }
            });
        });
    };
    TaskCancelManager.prototype.add = function (plannedTask) {
        this.listenToCancelEvent(plannedTask.cancelEvent);
    };
    TaskCancelManager.prototype.listenToCancelEvent = function (cancelEvent) {
        var _this = this;
        if (this.cancelEvents.has(cancelEvent)) {
            return;
        }
        this.cancelEvents.add(cancelEvent);
        var listenerId = events_1.on(cancelEvent, function (evt) {
            var evtName = evt.name;
            events_1.off(evtName, listenerId);
            _this.cancelEvents.delete(evtName);
            _this.cancelByEventName(evtName)
                .then(function () {
                task_chain_1.TaskChain.finalize(evt.id, task_chain_1.TaskResultStatus.Ok);
            })
                .catch(function (err) {
                task_chain_1.TaskChain.finalize(evt.id, task_chain_1.TaskResultStatus.Error, err);
            });
        });
    };
    TaskCancelManager.prototype.cancelByEventName = function (eventName) {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, _a, tasks, tasks_1, tasks_1_1, task, e_1_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4, this.taskStore.getAllFilteredByCancelEvent(eventName)];
                    case 1:
                        tasks = _b.sent();
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 7, 8, 9]);
                        tasks_1 = __values(tasks), tasks_1_1 = tasks_1.next();
                        _b.label = 3;
                    case 3:
                        if (!!tasks_1_1.done) return [3, 6];
                        task = tasks_1_1.value;
                        return [4, this.cancelTask(task)];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5:
                        tasks_1_1 = tasks_1.next();
                        return [3, 3];
                    case 6: return [3, 9];
                    case 7:
                        e_1_1 = _b.sent();
                        e_1 = { error: e_1_1 };
                        return [3, 9];
                    case 8:
                        try {
                            if (tasks_1_1 && !tasks_1_1.done && (_a = tasks_1.return)) _a.call(tasks_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7];
                    case 9: return [2];
                }
            });
        });
    };
    TaskCancelManager.prototype.cancelTask = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            var err_1, id, name_1, startAt, interval, recurrent;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 5, , 6]);
                        if (!(task.planningType === planned_task_1.PlanningType.Scheduled)) return [3, 2];
                        return [4, this.getTaskScheduler().cancel(task.id)];
                    case 1:
                        _a.sent();
                        return [3, 4];
                    case 2: return [4, this.taskStore.delete(task.id)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [3, 6];
                    case 5:
                        err_1 = _a.sent();
                        id = task.id, name_1 = task.name, startAt = task.startAt, interval = task.interval, recurrent = task.recurrent;
                        this.logger.error("Error canceling task: " + JSON.stringify({
                            id: id,
                            name: name_1,
                            startAt: startAt,
                            interval: interval,
                            recurrent: recurrent,
                        }) + " -> " + err_1);
                        return [3, 6];
                    case 6: return [2];
                }
            });
        });
    };
    TaskCancelManager.prototype.getTaskScheduler = function () {
        if (!this.taskScheduler) {
            this.taskScheduler = time_based_1.taskScheduler();
        }
        return this.taskScheduler;
    };
    return TaskCancelManager;
}());
exports.TaskCancelManager = TaskCancelManager;
exports.taskCancelManager = new TaskCancelManager();
//# sourceMappingURL=cancel-manager.js.map