"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var provider_1 = require("../provider");
var TaskGraphBrowser = (function () {
    function TaskGraphBrowser(taskProvider) {
        if (taskProvider === void 0) { taskProvider = provider_1.getTask; }
        this.taskProvider = taskProvider;
        this.entries = new Map();
    }
    TaskGraphBrowser.prototype.addEntry = function (invocationEvent, runnableTask) {
        if (!this.entries.has(invocationEvent)) {
            this.entries.set(invocationEvent, []);
        }
        this.entries.get(invocationEvent).push(runnableTask);
    };
    TaskGraphBrowser.prototype.getTriggeredBy = function (invocationEvent) {
        if (!this.entries.has(invocationEvent)) {
            return [];
        }
        return this.entries.get(invocationEvent);
    };
    TaskGraphBrowser.prototype.getUniques = function () {
        var e_1, _a, e_2, _b;
        var uniques = new Set();
        try {
            for (var _c = __values(this.entries.values()), _d = _c.next(); !_d.done; _d = _c.next()) {
                var runnableTasks = _d.value;
                try {
                    for (var runnableTasks_1 = __values(runnableTasks), runnableTasks_1_1 = runnableTasks_1.next(); !runnableTasks_1_1.done; runnableTasks_1_1 = runnableTasks_1.next()) {
                        var runnableTask = runnableTasks_1_1.value;
                        uniques.add(runnableTask);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (runnableTasks_1_1 && !runnableTasks_1_1.done && (_b = runnableTasks_1.return)) _b.call(runnableTasks_1);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return __spread(uniques);
    };
    TaskGraphBrowser.prototype.any = function (matcher) {
        var e_3, _a;
        try {
            for (var _b = __values(this.getUniques()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var runnableTask = _c.value;
                var instance = this.taskProvider(runnableTask.name);
                var matches = matcher(__assign({}, runnableTask, { instance: instance }));
                if (matches) {
                    return true;
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return false;
    };
    TaskGraphBrowser.prototype.anyFrom = function (eventName, matcher) {
        var memory = new Map();
        try {
            return this.memoizedAnyFrom(eventName, matcher, memory);
        }
        catch (err) {
            if (err instanceof RangeError) {
                throw new Error("Task graphs with cycles are not supported");
            }
            else {
                throw err;
            }
        }
    };
    TaskGraphBrowser.prototype.depict = function () {
        var _this = this;
        var rootEvents = this.getRootEvents();
        return rootEvents.map(function (eventName) { return ({
            trigger: eventName,
            tasks: _this.walkFrom(eventName),
        }); });
    };
    TaskGraphBrowser.prototype.walkFrom = function (eventName) {
        var memory = new Map();
        try {
            return this.memoizedWalk(eventName, memory);
        }
        catch (err) {
            if (err instanceof RangeError) {
                throw new Error("Task graphs with cycles are not supported");
            }
            else {
                throw err;
            }
        }
    };
    TaskGraphBrowser.prototype.getRootEvents = function () {
        var e_4, _a, e_5, _b, e_6, _c;
        var rootEvents = new Set(this.entries.keys());
        try {
            for (var _d = __values(this.entries.values()), _e = _d.next(); !_e.done; _e = _d.next()) {
                var runnableTasks = _e.value;
                try {
                    for (var runnableTasks_2 = __values(runnableTasks), runnableTasks_2_1 = runnableTasks_2.next(); !runnableTasks_2_1.done; runnableTasks_2_1 = runnableTasks_2.next()) {
                        var runnableTask = runnableTasks_2_1.value;
                        var task = this.taskProvider(runnableTask.name);
                        try {
                            for (var _f = __values(task.outputEventNames), _g = _f.next(); !_g.done; _g = _f.next()) {
                                var outputEvent = _g.value;
                                if (rootEvents.has(outputEvent)) {
                                    rootEvents.delete(outputEvent);
                                }
                            }
                        }
                        catch (e_6_1) { e_6 = { error: e_6_1 }; }
                        finally {
                            try {
                                if (_g && !_g.done && (_c = _f.return)) _c.call(_f);
                            }
                            finally { if (e_6) throw e_6.error; }
                        }
                    }
                }
                catch (e_5_1) { e_5 = { error: e_5_1 }; }
                finally {
                    try {
                        if (runnableTasks_2_1 && !runnableTasks_2_1.done && (_b = runnableTasks_2.return)) _b.call(runnableTasks_2);
                    }
                    finally { if (e_5) throw e_5.error; }
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) _a.call(_d);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return __spread(rootEvents);
    };
    TaskGraphBrowser.prototype.memoizedAnyFrom = function (eventName, matcher, memory) {
        var e_7, _a, e_8, _b;
        if (!this.entries.has(eventName)) {
            return false;
        }
        if (memory.has(eventName)) {
            return memory.get(eventName);
        }
        var runnableTasks = this.entries.get(eventName);
        try {
            for (var runnableTasks_3 = __values(runnableTasks), runnableTasks_3_1 = runnableTasks_3.next(); !runnableTasks_3_1.done; runnableTasks_3_1 = runnableTasks_3.next()) {
                var runnableTask = runnableTasks_3_1.value;
                var instance = this.taskProvider(runnableTask.name);
                var matches = matcher(__assign({}, runnableTask, { instance: instance }));
                if (matches) {
                    return true;
                }
                try {
                    for (var _c = __values(instance.outputEventNames), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var outputEvent = _d.value;
                        var childMatches = this.memoizedAnyFrom(outputEvent, matcher, memory);
                        if (childMatches) {
                            return true;
                        }
                    }
                }
                catch (e_8_1) { e_8 = { error: e_8_1 }; }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return)) _b.call(_c);
                    }
                    finally { if (e_8) throw e_8.error; }
                }
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (runnableTasks_3_1 && !runnableTasks_3_1.done && (_a = runnableTasks_3.return)) _a.call(runnableTasks_3);
            }
            finally { if (e_7) throw e_7.error; }
        }
        return false;
    };
    TaskGraphBrowser.prototype.memoizedWalk = function (eventName, memory) {
        var _this = this;
        if (!this.entries.has(eventName)) {
            return [];
        }
        if (memory.has(eventName)) {
            return memory.get(eventName);
        }
        var runnableTasks = this.entries.get(eventName);
        var eventTriggeredTasks = runnableTasks.map(function (runnableTask) { return (__assign({}, runnableTask, { outputs: _this.walkOutputs(runnableTask, memory) })); });
        memory.set(eventName, eventTriggeredTasks);
        return eventTriggeredTasks;
    };
    TaskGraphBrowser.prototype.walkOutputs = function (runnableTask, memory) {
        var _this = this;
        var task = this.taskProvider(runnableTask.name);
        return task.outputEventNames.map(function (outputEvent) { return ({
            trigger: outputEvent,
            tasks: _this.memoizedWalk(outputEvent, memory),
        }); });
    };
    return TaskGraphBrowser;
}());
exports.TaskGraphBrowser = TaskGraphBrowser;
exports.taskGraphBrowser = new TaskGraphBrowser();
//# sourceMappingURL=browser.js.map