"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var time_based_1 = require("../schedulers/time-based");
var instant_task_runner_1 = require("../schedulers/immediate/instant-task-runner");
var planned_tasks_store_1 = require("../../persistence/planned-tasks-store");
var cancel_manager_1 = require("../cancel-manager");
var browser_1 = require("../graph/browser");
var provider_1 = require("../provider");
var task_chain_1 = require("../task-chain");
var parallelizer_1 = require("./parallelizer");
var TaskPlanner = (function () {
    function TaskPlanner(taskScheduler, taskRunner, taskStore, cancelManager, parallelizer) {
        if (taskRunner === void 0) { taskRunner = new instant_task_runner_1.InstantTaskRunner(planned_tasks_store_1.plannedTasksDB); }
        if (taskStore === void 0) { taskStore = planned_tasks_store_1.plannedTasksDB; }
        if (cancelManager === void 0) { cancelManager = cancel_manager_1.taskCancelManager; }
        if (parallelizer === void 0) { parallelizer = new parallelizer_1.TaskPlannerParallelizer(browser_1.taskGraphBrowser); }
        this.taskScheduler = taskScheduler;
        this.taskRunner = taskRunner;
        this.taskStore = taskStore;
        this.cancelManager = cancelManager;
        this.parallelizer = parallelizer;
    }
    TaskPlanner.prototype.plan = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var invocationEvent, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        invocationEvent = this.parallelizer.spawnChildEvent(dispatchableEvent);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        provider_1.checkIfTaskExists(runnableTask.name);
                        return [4, (runnableTask.interval > 0 || runnableTask.startAt !== -1
                                ? this.planScheduled(runnableTask, invocationEvent)
                                : this.planImmediate(runnableTask, invocationEvent))];
                    case 2: return [2, _a.sent()];
                    case 3:
                        err_1 = _a.sent();
                        emitTaskChainFinished(invocationEvent, err_1);
                        throw err_1;
                    case 4: return [2];
                }
            });
        });
    };
    TaskPlanner.prototype.planImmediate = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var existedBefore, plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskStore.get(runnableTask)];
                    case 1:
                        existedBefore = _a.sent();
                        return [4, this.taskRunner.run(runnableTask, dispatchableEvent)];
                    case 2:
                        plannedTask = _a.sent();
                        if (!existedBefore) {
                            this.cancelManager.add(plannedTask);
                        }
                        return [2, plannedTask];
                }
            });
        });
    };
    TaskPlanner.prototype.planScheduled = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var possibleExisting, plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskStore.get(runnableTask)];
                    case 1:
                        possibleExisting = _a.sent();
                        if (possibleExisting) {
                            emitTaskChainFinished(dispatchableEvent);
                            return [2, possibleExisting];
                        }
                        return [4, this.getTaskScheduler().schedule(runnableTask)];
                    case 2:
                        plannedTask = _a.sent();
                        emitTaskChainFinished(dispatchableEvent);
                        this.cancelManager.add(plannedTask);
                        return [2, plannedTask];
                }
            });
        });
    };
    TaskPlanner.prototype.getTaskScheduler = function () {
        if (!this.taskScheduler) {
            this.taskScheduler = time_based_1.taskScheduler();
        }
        return this.taskScheduler;
    };
    return TaskPlanner;
}());
exports.TaskPlanner = TaskPlanner;
function emitTaskChainFinished(dispatchableEvent, error) {
    if (!dispatchableEvent) {
        return;
    }
    var status = error ? task_chain_1.TaskResultStatus.Error : task_chain_1.TaskResultStatus.Ok;
    task_chain_1.TaskChain.finalize(dispatchableEvent.id, status, error);
}
//# sourceMappingURL=index.js.map