"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("../../events");
var task_chain_1 = require("../task-chain");
var logger_1 = require("../../utils/logger");
var TaskPlannerParallelizer = (function () {
    function TaskPlannerParallelizer(graphBrowser) {
        this.graphBrowser = graphBrowser;
        this.triggerData = new Map();
    }
    TaskPlannerParallelizer.prototype.spawnChildEvent = function (parent) {
        var siblingTasks = this.graphBrowser.getTriggeredBy(parent.name);
        if (siblingTasks.length < 2) {
            return parent;
        }
        var triggerId = parent.name + "#" + parent.id;
        if (!this.triggerData.has(triggerId)) {
            var timeoutId = this.setupTimeoutPropagation(triggerId);
            this.triggerData.set(triggerId, createEventRiseData(siblingTasks.length, timeoutId));
        }
        var childEvent = events_1.createEvent(parent.name, {
            data: parent.data,
            expirationTimestamp: parent.expirationTimestamp,
        });
        var childTasks = this.triggerData.get(triggerId).childTasks;
        childTasks.eventIds.add(childEvent.id);
        this.setupTaskFinalizationSink(childEvent.id, triggerId);
        this.getLogger().info("Child event (name=" + childEvent.name + ", id=" + childEvent.id + ") spawned (invocationId=" + parent.id + ")");
        return childEvent;
    };
    TaskPlannerParallelizer.prototype.setupTimeoutPropagation = function (triggerId) {
        var _this = this;
        var timeoutId = events_1.on(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, function (evt) {
            var e_1, _a;
            var _b = __read(triggerId.split("#"), 2), eventId = _b[1];
            if (evt.id !== eventId) {
                return;
            }
            events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, timeoutId);
            if (!_this.triggerData.has(triggerId)) {
                return;
            }
            var triggerData = _this.triggerData.get(triggerId);
            triggerData.didTimedOut = true;
            var childTasks = triggerData.childTasks;
            try {
                for (var _c = __values(childTasks.eventIds), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var childEventId = _d.value;
                    var timeoutEvent = events_1.createEvent(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, { id: childEventId });
                    events_1.emit(timeoutEvent);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
        return timeoutId;
    };
    TaskPlannerParallelizer.prototype.setupTaskFinalizationSink = function (eventId, triggerId) {
        var _this = this;
        var listenerId = events_1.on(events_1.TaskDispatcherEvent.TaskChainFinished, function (evt) {
            if (evt.id !== eventId) {
                return;
            }
            events_1.off(events_1.TaskDispatcherEvent.TaskChainFinished, listenerId);
            if (!_this.triggerData.has(triggerId)) {
                return;
            }
            var triggerData = _this.triggerData.get(triggerId);
            var childTasks = triggerData.childTasks;
            childTasks.eventIds.delete(eventId);
            childTasks.count--;
            if (childTasks.count === 0) {
                var _a = __read(triggerId.split("#"), 2), parentId = _a[1];
                var didTimedOut = triggerData.didTimedOut;
                if (!didTimedOut) {
                    events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, triggerData.timeoutId);
                }
                _this.triggerData.delete(triggerId);
                task_chain_1.TaskChain.finalize(parentId, didTimedOut ? task_chain_1.TaskResultStatus.Cancelled : task_chain_1.TaskResultStatus.Ok);
            }
        });
    };
    TaskPlannerParallelizer.prototype.getLogger = function () {
        if (!this.logger) {
            this.logger = logger_1.getLogger("TaskPlannerParallelizer");
        }
        return this.logger;
    };
    return TaskPlannerParallelizer;
}());
exports.TaskPlannerParallelizer = TaskPlannerParallelizer;
function createEventRiseData(taskCount, timeoutId) {
    return {
        childTasks: {
            count: taskCount,
            eventIds: new Set(),
        },
        didTimedOut: false,
        timeoutId: timeoutId,
    };
}
//# sourceMappingURL=parallelizer.js.map