"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var uuid_1 = require("../../utils/uuid");
var time_1 = require("../../utils/time");
var PlanningType;
(function (PlanningType) {
    PlanningType["Scheduled"] = "scheduled";
    PlanningType["Immediate"] = "immediate";
})(PlanningType = exports.PlanningType || (exports.PlanningType = {}));
var SchedulerType;
(function (SchedulerType) {
    SchedulerType["None"] = "none";
    SchedulerType["Alarm"] = "alarm";
})(SchedulerType = exports.SchedulerType || (exports.SchedulerType = {}));
var PlannedTask = (function () {
    function PlannedTask(planningType, schedulerType, task, id, createdAt, lastRun, errorCount, timeoutCount) {
        if (id === void 0) { id = uuid_1.uuid(); }
        if (createdAt === void 0) { createdAt = time_1.now(); }
        if (lastRun === void 0) { lastRun = -1; }
        if (errorCount === void 0) { errorCount = 0; }
        if (timeoutCount === void 0) { timeoutCount = 0; }
        this.planningType = planningType;
        this.schedulerType = schedulerType;
        this.id = id;
        this.createdAt = createdAt;
        this.lastRun = lastRun;
        this.errorCount = errorCount;
        this.timeoutCount = timeoutCount;
        this.name = task.name;
        this.startAt = task.startAt;
        this.interval = task.interval;
        this.recurrent = task.recurrent;
        this.params = task.params;
        this.cancelEvent = task.cancelEvent;
    }
    Object.defineProperty(PlannedTask.prototype, "lastUpdate", {
        get: function () {
            return this.lastRun !== -1 ? this.lastRun : this.createdAt;
        },
        enumerable: true,
        configurable: true
    });
    PlannedTask.prototype.nextRun = function (currentTime) {
        if (currentTime === void 0) { currentTime = time_1.now(); }
        if (this.startAt !== -1 && currentTime < this.startAt) {
            return this.startAt - currentTime;
        }
        return this.interval - (currentTime - this.lastUpdate);
    };
    return PlannedTask;
}());
exports.PlannedTask = PlannedTask;
//# sourceMappingURL=planned-task.js.map