import { EventReceiver, DispatchableEvent } from "../../events";
import { RunnableTask } from ".";
import { TimeUnit } from "../../utils/time-converter";
import { TaskParams } from "../task";
import { TaskPlanner } from "../planner";
import { PlannedTask } from "../planner/planned-task";
export interface ReadyRunnableTaskBuilder extends EventReceiver {
    build(): RunnableTask;
    plan(dispatchableEvent?: DispatchableEvent): Promise<PlannedTask>;
}
interface ScheduledRunnableTaskBuilder extends ReadyRunnableTaskBuilder {
    cancelOn(eventName: string): ReadyRunnableTaskBuilder;
}
interface DelayedRunnableTaskBuilder extends ScheduledRunnableTaskBuilder {
    every(time: number, timeUnit?: TimeUnit): ScheduledRunnableTaskBuilder;
}
export interface RunnableTaskBuilder extends ReadyRunnableTaskBuilder {
    now(): ReadyRunnableTaskBuilder;
    every(time: number, timeUnit?: TimeUnit): ScheduledRunnableTaskBuilder;
    in(time: number, timeUnit?: TimeUnit): ScheduledRunnableTaskBuilder;
    at(date: Date): DelayedRunnableTaskBuilder;
}
export declare class RunnableTaskBuilderImpl implements RunnableTaskBuilder {
    private taskName;
    private params;
    private taskPlanner?;
    private startAt;
    private interval;
    private recurrent;
    private cancelEvent;
    private logger;
    constructor(taskName: string, params: TaskParams, taskPlanner?: TaskPlanner);
    now(): ReadyRunnableTaskBuilder;
    every(time: number, timeUnit?: TimeUnit): ScheduledRunnableTaskBuilder;
    in(time: number, timeUnit?: TimeUnit): ScheduledRunnableTaskBuilder;
    at(date: Date): DelayedRunnableTaskBuilder;
    cancelOn(eventName: string): ReadyRunnableTaskBuilder;
    build(): RunnableTask;
    plan(dispatchableEvent?: DispatchableEvent): Promise<PlannedTask>;
    onReceive(dispatchableEvent: DispatchableEvent): void;
}
export {};
