"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("../../events");
var time_converter_1 = require("../../utils/time-converter");
var logger_1 = require("../../utils/logger");
var RunnableTaskBuilderImpl = (function () {
    function RunnableTaskBuilderImpl(taskName, params, taskPlanner) {
        this.taskName = taskName;
        this.params = params;
        this.taskPlanner = taskPlanner;
        this.startAt = -1;
        this.interval = 0;
        this.recurrent = false;
        this.cancelEvent = events_1.TaskDispatcherEvent.DefaultCancelEvent;
        this.logger = logger_1.getLogger("RunnableTaskBuilder");
    }
    RunnableTaskBuilderImpl.prototype.now = function () {
        this.startAt = -1;
        this.interval = 0;
        this.recurrent = false;
        return this;
    };
    RunnableTaskBuilderImpl.prototype.every = function (time, timeUnit) {
        var seconds = timeUnit ? time_converter_1.toSeconds(time, timeUnit) : time;
        this.interval = seconds;
        this.recurrent = true;
        return this;
    };
    RunnableTaskBuilderImpl.prototype.in = function (time, timeUnit) {
        var seconds = timeUnit ? time_converter_1.toSeconds(time, timeUnit) : time;
        this.interval = seconds;
        this.recurrent = false;
        return this;
    };
    RunnableTaskBuilderImpl.prototype.at = function (date) {
        if (date > new Date()) {
            this.startAt = date.getTime();
        }
        this.recurrent = false;
        return this;
    };
    RunnableTaskBuilderImpl.prototype.cancelOn = function (eventName) {
        this.cancelEvent = eventName;
        return this;
    };
    RunnableTaskBuilderImpl.prototype.build = function () {
        return {
            name: this.taskName,
            startAt: this.startAt,
            interval: this.interval,
            recurrent: this.recurrent,
            cancelEvent: this.cancelEvent,
            params: this.params,
        };
    };
    RunnableTaskBuilderImpl.prototype.plan = function (dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var runnableTask;
            return __generator(this, function (_a) {
                runnableTask = this.build();
                try {
                    return [2, this.taskPlanner.plan(runnableTask, dispatchableEvent)];
                }
                catch (err) {
                    throw new Error("Could not plan (" + JSON.stringify(runnableTask) + "). Reason: " + err);
                }
                return [2];
            });
        });
    };
    RunnableTaskBuilderImpl.prototype.onReceive = function (dispatchableEvent) {
        var _this = this;
        this.plan(dispatchableEvent)
            .then(function (plannedTask) {
            _this.logger.info("Task planned: " + JSON.stringify(plannedTask));
        })
            .catch(function (err) {
            _this.logger.error(err);
        });
    };
    return RunnableTaskBuilderImpl;
}());
exports.RunnableTaskBuilderImpl = RunnableTaskBuilderImpl;
//# sourceMappingURL=builder.js.map