"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("../../events");
var single_task_runner_1 = require("./single-task-runner");
var logger_1 = require("../../utils/logger");
var BatchTaskRunner = (function () {
    function BatchTaskRunner(taskStore) {
        this.taskRunner = new single_task_runner_1.SingleTaskRunner(taskStore);
        this.logger = logger_1.getLogger("BatchTaskRunner");
    }
    BatchTaskRunner.prototype.run = function (plannedTasks, startEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, Promise.all(plannedTasks.map(function (plannedTask) {
                            return _this.runTaskWithCustomStartEvent(plannedTask, startEvent);
                        }))];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    BatchTaskRunner.prototype.runTaskWithCustomStartEvent = function (plannedTask, batchStartEvent) {
        var startEvent = events_1.createEvent(events_1.TaskDispatcherEvent.TaskExecutionStarted, {
            expirationTimestamp: batchStartEvent.expirationTimestamp,
        });
        var listenerId = events_1.on(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, function (evt) {
            if (evt.id === batchStartEvent.id) {
                events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                events_1.emit(events_1.createEvent(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, {
                    id: startEvent.id,
                }));
            }
        });
        this.logger.info("Child event (name=" + startEvent.name + ", id=" + startEvent.id + ") spawned (invocationId=" + batchStartEvent.id + ")");
        return this.taskRunner.run(plannedTask, startEvent);
    };
    return BatchTaskRunner;
}());
exports.BatchTaskRunner = BatchTaskRunner;
//# sourceMappingURL=batch-task-runner.js.map