"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var provider_1 = require("../provider");
var events_1 = require("../../events");
var logger_1 = require("../../utils/logger");
var time_1 = require("../../utils/time");
var last_recurrent_run_1 = require("./last-recurrent-run");
var FAILURE_THRESHOLD = 3;
var SingleTaskRunner = (function () {
    function SingleTaskRunner(taskStore) {
        this.taskStore = taskStore;
        this.logger = logger_1.getLogger("SingleTaskRunner");
    }
    SingleTaskRunner.prototype.run = function (plannedTask, startEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var name, id, params, task, lastRunOffset, parameterizedTask, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        name = plannedTask.name, id = plannedTask.id, params = plannedTask.params;
                        task = provider_1.getTask(name);
                        lastRunOffset = plannedTask.recurrent
                            ? last_recurrent_run_1.lastRecurrentRun.timeSince()
                            : 0;
                        return [4, this.taskStore.updateLastRun(id, time_1.now() - lastRunOffset)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 6]);
                        parameterizedTask = new ParameterizedTask(task, params, startEvent);
                        return [4, this.runWithTimeout(id, parameterizedTask, startEvent.id)];
                    case 3:
                        _a.sent();
                        return [3, 6];
                    case 4:
                        error_1 = _a.sent();
                        return [4, this.taskStore.increaseErrorCount(id)];
                    case 5:
                        _a.sent();
                        return [3, 6];
                    case 6:
                        if (!(!plannedTask.recurrent && plannedTask.interval > 0)) return [3, 8];
                        return [4, this.handleOneShotTask(plannedTask.id)];
                    case 7:
                        _a.sent();
                        _a.label = 8;
                    case 8: return [2];
                }
            });
        });
    };
    SingleTaskRunner.prototype.runWithTimeout = function (id, task, startEventId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var listenerId = events_1.on(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, function (evt) {
                if (evt.id === startEventId) {
                    events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                    task.cancel();
                    _this.taskStore.increaseTimeoutCount(id).then(function () { return resolve(); });
                }
            });
            var taskAlreadyRun = false;
            var taskChainFinished = false;
            _this.waitForTaskChainToFinish(startEventId).then(function () {
                taskChainFinished = true;
                if (taskAlreadyRun) {
                    resolve();
                }
            });
            task
                .run()
                .then(function () {
                taskAlreadyRun = true;
                events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                if (taskChainFinished) {
                    resolve();
                }
            })
                .catch(function (err) {
                events_1.off(events_1.TaskDispatcherEvent.TaskExecutionTimedOut, listenerId);
                reject(err);
            });
        });
    };
    SingleTaskRunner.prototype.waitForTaskChainToFinish = function (startEventId) {
        return new Promise(function (resolve) {
            var listenerId = events_1.on(events_1.TaskDispatcherEvent.TaskChainFinished, function (chainFinishedEvt) {
                if (chainFinishedEvt.id === startEventId) {
                    events_1.off(events_1.TaskDispatcherEvent.TaskChainFinished, listenerId);
                    resolve();
                }
            });
        });
    };
    SingleTaskRunner.prototype.handleOneShotTask = function (id) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, errorCount, timeoutCount;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4, this.taskStore.get(id)];
                    case 1:
                        _a = _b.sent(), errorCount = _a.errorCount, timeoutCount = _a.timeoutCount;
                        if (!(errorCount === 0 && timeoutCount === 0)) return [3, 3];
                        return [4, this.taskStore.delete(id)];
                    case 2:
                        _b.sent();
                        return [3, 5];
                    case 3:
                        if (!(errorCount > FAILURE_THRESHOLD ||
                            timeoutCount > FAILURE_THRESHOLD)) return [3, 5];
                        this.logger.warn("One-shot planned task id:" + id + " discarded after {errorCount:" + errorCount + ", timeoutCount:" + timeoutCount + "}");
                        return [4, this.taskStore.delete(id)];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5: return [2];
                }
            });
        });
    };
    return SingleTaskRunner;
}());
exports.SingleTaskRunner = SingleTaskRunner;
var ParameterizedTask = (function () {
    function ParameterizedTask(task, taskParams, startEvent) {
        this.task = task;
        this.taskParams = taskParams;
        this.startEvent = startEvent;
    }
    ParameterizedTask.prototype.run = function () {
        return this.task.run(this.taskParams, this.startEvent);
    };
    ParameterizedTask.prototype.cancel = function () {
        this.task.cancel();
    };
    return ParameterizedTask;
}());
//# sourceMappingURL=single-task-runner.js.map