"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("tns-core-modules/application/application");
var intents_android_1 = require("../../intents.android");
var abstract_alarm_manager_android_1 = require("../abstract-alarm-manager.android");
var power_savings_manager_android_1 = require("../power-savings-manager.android");
var logger_1 = require("../../../../../../utils/logger");
var planning_timestamp_1 = require("../../../planning-timestamp");
var time_1 = require("../../../../../../utils/time");
var BATTERY_SAVINGS_THRESHOLD = 15 * 60 * 1000;
var AndroidAlarmManager = (function (_super) {
    __extends(AndroidAlarmManager, _super);
    function AndroidAlarmManager(osAlarmManager, powerManager, sdkVersion) {
        if (osAlarmManager === void 0) { osAlarmManager = application_1.android.context.getSystemService(android.content.Context.ALARM_SERVICE); }
        if (powerManager === void 0) { powerManager = power_savings_manager_android_1.getPowerSavingsManager(); }
        if (sdkVersion === void 0) { sdkVersion = android.os.Build.VERSION.SDK_INT; }
        var _this = _super.call(this, osAlarmManager, intents_android_1.createAlarmReceiverIntent(application_1.android.context), logger_1.getLogger("AndroidAlarmManager")) || this;
        _this.powerManager = powerManager;
        _this.sdkVersion = sdkVersion;
        return _this;
    }
    AndroidAlarmManager.prototype.set = function (interval) {
        if (interval < BATTERY_SAVINGS_THRESHOLD) {
            this.checkPowerSavings();
        }
        if (this.alarmUp) {
            this.cancel();
        }
        var alarmType = android.app.AlarmManager.RTC_WAKEUP;
        var triggerAtMillis = time_1.now() + interval;
        var pendingIntent = this.getPendingIntent();
        if (this.sdkVersion >= 23) {
            this.osAlarmManager.setExactAndAllowWhileIdle(alarmType, triggerAtMillis, pendingIntent);
        }
        else if (this.sdkVersion >= 19) {
            this.osAlarmManager.setExact(alarmType, triggerAtMillis, pendingIntent);
        }
        else {
            this.osAlarmManager.set(alarmType, triggerAtMillis, pendingIntent);
        }
        planning_timestamp_1.planningTimestamp.updateCurrent();
        this.logger.info("Alarm will be triggered in " + interval + " ms");
    };
    AndroidAlarmManager.prototype.checkPowerSavings = function () {
        if (this.powerManager.areDisabled()) {
            return;
        }
        this.powerManager.requestDeactivation();
    };
    return AndroidAlarmManager;
}(abstract_alarm_manager_android_1.AbstractAlarmManager));
exports.AndroidAlarmManager = AndroidAlarmManager;
//# sourceMappingURL=manager.android.js.map