"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var manager_1 = require("../../../../../manager");
var manager_android_1 = require("./manager.android");
var planned_tasks_store_1 = require("../../../../../../persistence/planned-tasks-store");
var foreground_checker_1 = require("../../../../../foreground-checker");
var intents_android_1 = require("../../intents.android");
var planned_task_1 = require("../../../../../planner/planned-task");
var logger_1 = require("../../../../../../utils/logger");
var time_1 = require("../../../../../../utils/time");
var last_recurrent_run_1 = require("../../../../../runners/last-recurrent-run");
var MIN_INTERVAL = 59000;
var AlarmReceiver = (function () {
    function AlarmReceiver() {
    }
    AlarmReceiver.prototype.onReceive = function (context, intent) {
        var _this = this;
        last_recurrent_run_1.lastRecurrentRun.updateLast();
        this.logger = logger_1.getLogger("AlarmReceiver");
        this.logger.info("Alarm triggered");
        this.timeOffset = 30000;
        this.currentTime = time_1.now();
        this.taskManager = new manager_1.TaskManager(planned_task_1.PlanningType.Scheduled, planned_tasks_store_1.plannedTasksDB, new foreground_checker_1.ForegroundChecker(), this.timeOffset, this.currentTime);
        this.alarmManager = new manager_android_1.AndroidAlarmManager();
        this.handleAlarmTrigger(context)
            .then(function () {
            _this.logger.debug("Alarm handled");
        })
            .catch(function (err) {
            _this.logger.error(err);
        });
    };
    AlarmReceiver.prototype.handleAlarmTrigger = function (context) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.rescheduleIfNeeded()];
                    case 1:
                        _a.sent();
                        return [4, this.startTaskRunnerService(context)];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    AlarmReceiver.prototype.rescheduleIfNeeded = function () {
        return __awaiter(this, void 0, void 0, function () {
            var willContinue, nextInterval;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskManager.willContinue()];
                    case 1:
                        willContinue = _a.sent();
                        if (!willContinue) return [3, 3];
                        return [4, this.taskManager.nextInterval()];
                    case 2:
                        nextInterval = _a.sent();
                        nextInterval = nextInterval > MIN_INTERVAL ? nextInterval : MIN_INTERVAL;
                        this.alarmManager.set(nextInterval);
                        this.logger.info("Next alarm will be run in: " + nextInterval);
                        return [3, 4];
                    case 3:
                        this.logger.info("Won't reschedule");
                        _a.label = 4;
                    case 4: return [2];
                }
            });
        });
    };
    AlarmReceiver.prototype.startTaskRunnerService = function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var tasksToRun, requiresForeground;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskManager.tasksToRun()];
                    case 1:
                        tasksToRun = _a.sent();
                        if (!(tasksToRun.length > 0)) return [3, 3];
                        return [4, this.taskManager.requiresForeground()];
                    case 2:
                        requiresForeground = _a.sent();
                        this.startAlarmRunnerService(context, requiresForeground);
                        return [3, 4];
                    case 3:
                        this.logger.warn("Alarm triggered without tasks to run!");
                        _a.label = 4;
                    case 4: return [2];
                }
            });
        });
    };
    AlarmReceiver.prototype.startAlarmRunnerService = function (context, inForeground) {
        var startRunnerService = intents_android_1.createAlarmRunnerServiceIntent(context, {
            runInForeground: inForeground,
            timeOffset: this.timeOffset,
            invocationTime: this.currentTime,
        });
        if (inForeground) {
            androidx.core.content.ContextCompat.startForegroundService(context, startRunnerService);
        }
        else {
            context.startService(startRunnerService);
        }
    };
    return AlarmReceiver;
}());
exports.AlarmReceiver = AlarmReceiver;
var _alarmReceiver;
function getAlarmReceiver() {
    if (!_alarmReceiver) {
        _alarmReceiver = new AlarmReceiver();
    }
    return _alarmReceiver;
}
exports.getAlarmReceiver = getAlarmReceiver;
//# sourceMappingURL=receiver.android.js.map