"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var planned_task_1 = require("../../../planner/planned-task");
var scheduler_android_1 = require("./alarms/alarm/scheduler.android");
var planned_tasks_store_1 = require("../../../../persistence/planned-tasks-store");
var provider_1 = require("../../../provider");
var time_converter_1 = require("../../../../utils/time-converter");
var AndroidTaskScheduler = (function () {
    function AndroidTaskScheduler(alarmScheduler, tasksStore) {
        if (alarmScheduler === void 0) { alarmScheduler = new scheduler_android_1.AndroidAlarmScheduler(); }
        if (tasksStore === void 0) { tasksStore = planned_tasks_store_1.plannedTasksDB; }
        this.alarmScheduler = alarmScheduler;
        this.tasksStore = tasksStore;
    }
    AndroidTaskScheduler.prototype.setup = function () {
        this.alarmScheduler.setup();
    };
    AndroidTaskScheduler.prototype.schedule = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            var time, taskToSchedule;
            return __generator(this, function (_a) {
                provider_1.checkIfTaskExists(task.name);
                time = task.interval;
                taskToSchedule = __assign({}, task, { interval: time * 1000 });
                if (time < 1) {
                    throw new Error("Scheduled tasks cannot run in less than 1 second intervals. Use an immediate task instead.\n\n        Keep in mind that recurrent immediate tasks are hard to implement and strongly discouraged.");
                }
                if (time >= 1 && time < time_converter_1.toSeconds(1, "minutes")) {
                    throw new Error("Running tasks with intervals ranging from 1 to 60 seconds is a foreseen feature and will be implemented in the future.\n\n        Keep in mind that executing this kind of tasks in a sustained way will drain system resources. Use them responsibly.");
                }
                else {
                    return [2, this.alarmScheduler.schedule(taskToSchedule)];
                }
                return [2];
            });
        });
    };
    AndroidTaskScheduler.prototype.cancel = function (plannedTaskId) {
        return __awaiter(this, void 0, void 0, function () {
            var task;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.tasksStore.get(plannedTaskId)];
                    case 1:
                        task = _a.sent();
                        switch (task.planningType) {
                            case planned_task_1.PlanningType.Scheduled:
                                return [2, this.alarmScheduler.cancel(plannedTaskId)];
                            default:
                                throw new Error("Method not implemented.");
                        }
                        return [2];
                }
            });
        });
    };
    return AndroidTaskScheduler;
}());
exports.AndroidTaskScheduler = AndroidTaskScheduler;
var _androidTaskScheduler;
function getAndroidTaskScheduler() {
    if (!_androidTaskScheduler) {
        _androidTaskScheduler = new AndroidTaskScheduler();
    }
    return _androidTaskScheduler;
}
exports.getAndroidTaskScheduler = getAndroidTaskScheduler;
//# sourceMappingURL=index.js.map