"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("tns-core-modules/utils/utils");
var intents_android_1 = require("./intents.android");
var logger_1 = require("../../../../utils/logger");
var AndroidNotification;
(function (AndroidNotification) {
    AndroidNotification[AndroidNotification["LocationUsage"] = 1000] = "LocationUsage";
})(AndroidNotification = exports.AndroidNotification || (exports.AndroidNotification = {}));
var _notificationChannels;
function notificationChannels(context) {
    if (!_notificationChannels) {
        var getString = createStringFetcher(context);
        var NotificationManagerCompat = androidx.core.app.NotificationManagerCompat;
        _notificationChannels = new Map();
        try {
            _notificationChannels.set(AndroidNotification.LocationUsage, {
                id: "LOCATION_USAGE",
                name: getString("task_dispatcher_channel_name"),
                description: getString("task_dispatcher_channel_description"),
                priority: NotificationManagerCompat.IMPORTANCE_LOW,
            });
        }
        catch (e) {
            logger().debug("No strings set for " + AndroidNotification.LocationUsage + " channel. Skipping...");
        }
    }
    return _notificationChannels;
}
function setupNotificationChannels(context) {
    var e_1, _a;
    var sdkInt = android.os.Build.VERSION.SDK_INT;
    if (sdkInt < 26) {
        return;
    }
    try {
        for (var _b = __values(notificationChannels(context).values()), _c = _b.next(); !_c.done; _c = _b.next()) {
            var channel = _c.value;
            setupNotificationChannel(context, channel);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
}
exports.setupNotificationChannels = setupNotificationChannels;
function createNotification(context, type) {
    if (!notificationChannels(context).has(type)) {
        throw new Error("Translation strings had not been setup for " + AndroidNotification[type] + " notification type. Please follow setup instructions.");
    }
    var notificationBuilder;
    switch (type) {
        case AndroidNotification.LocationUsage:
            var appLaunchIntent = intents_android_1.createAppLaunchIntent(context);
            var pendingIntent = android.app.PendingIntent.getActivity(context, 0, appLaunchIntent, 0);
            notificationBuilder = initializeNotificationBuilder(context, type, "task_dispatcher_notification_title", "task_dispatcher_notification_content").setContentIntent(pendingIntent);
            break;
    }
    return notificationBuilder ? notificationBuilder.build() : null;
}
exports.createNotification = createNotification;
function setupNotificationChannel(context, channel) {
    var notificationManager = context.getSystemService(android.app.NotificationManager.class);
    if (!notificationManager) {
        return;
    }
    var id = channel.id, name = channel.name, description = channel.description, priority = channel.priority;
    var ch = new android.app.NotificationChannel(id, name, priority);
    ch.setDescription(description);
    notificationManager.createNotificationChannel(ch);
}
function initializeNotificationBuilder(context, type, title, content) {
    var _a = notificationChannels(context).get(type), id = _a.id, priority = _a.priority;
    var iconId = context
        .getResources()
        .getIdentifier("icon", "drawable", context.getApplicationInfo().packageName);
    var getString = createStringFetcher(context);
    return new androidx.core.app.NotificationCompat.Builder(context, id)
        .setSmallIcon(iconId)
        .setContentTitle(getString(title))
        .setContentText(getString(content))
        .setPriority(priority);
}
function createStringFetcher(context) {
    return function (key) {
        return context.getResources().getString(utils_1.ad.resources.getStringId(key));
    };
}
var _logger;
function logger() {
    if (!_logger) {
        _logger = logger_1.getLogger("NotificationManager");
    }
    return _logger;
}
//# sourceMappingURL=notification-manager.android.js.map