import { DispatchableEvent } from "../events";
export declare abstract class Task {
    readonly name: string;
    readonly outputEventNames: Array<string>;
    private _taskConfig;
    private _taskParams;
    private _invocationEvent;
    private _name;
    private _executionHistory;
    private _cancelFunctions;
    private _logger;
    constructor(name: string, taskConfig?: TaskConfig);
    run(taskParams: TaskParams, invocationEvent: DispatchableEvent): Promise<void>;
    runsInBackground(): boolean;
    cancel(): void;
    checkIfCanRun(): Promise<void>;
    prepare(): Promise<void>;
    protected runAgainIn(seconds: number, params?: TaskParams): void;
    protected abstract onRun(taskParams: TaskParams, invocationEvent: DispatchableEvent): Promise<void | TaskOutcome>;
    protected setCancelFunction(f: CancelFunction): void;
    protected log(message: any): void;
    protected remainingTime(): number;
    private configureTask;
    private cancelParallelInvocation;
    private processTaskOutcome;
    private done;
    private isDone;
    private markAsDone;
    private emitEndEvent;
    private removeCancelFunction;
    private getLogger;
}
export interface TaskConfig {
    foreground?: boolean;
    outputEventNames?: Array<string>;
}
export interface TaskParams {
    [key: string]: any;
}
export interface TaskOutcome {
    eventName?: string;
    result?: any;
}
export declare type CancelFunction = () => void;
export declare type TaskDeferrer = (taskName: string, seconds: number, taskParams: TaskParams) => void;
export declare function setTaskDeferrer(taskDeferrer: TaskDeferrer): void;
