"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var task_dispatcher_common_1 = require("./task-dispatcher.common");
var browser_1 = require("./internal/tasks/graph/browser");
var common_1 = require("./internal/tasks/schedulers/time-based/common");
var android_1 = require("./internal/tasks/schedulers/time-based/android");
var common_2 = require("./internal/tasks/schedulers/event-driven/common");
var android_2 = require("./internal/tasks/schedulers/event-driven/android");
var boot_receiver_android_1 = require("./internal/tasks/schedulers/time-based/android/boot-receiver.android");
var receiver_android_1 = require("./internal/tasks/schedulers/time-based/android/alarms/alarm/receiver.android");
var runner_service_android_1 = require("./internal/tasks/schedulers/time-based/android/alarms/alarm/runner-service.android");
var receiver_android_2 = require("./internal/tasks/schedulers/time-based/android/alarms/watchdog/receiver.android");
var runner_service_android_2 = require("./internal/tasks/schedulers/event-driven/android/runner-service.android");
var power_savings_manager_android_1 = require("./internal/tasks/schedulers/time-based/android/alarms/power-savings-manager.android");
var BATTERY_OPTIMIZATIONS_THRESHOLD = 15 * 60;
var TaskDispatcher = (function (_super) {
    __extends(TaskDispatcher, _super);
    function TaskDispatcher() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TaskDispatcher.prototype.init = function (appTasks, appTaskGraph, config) {
        common_1.setTaskSchedulerCreator(function () { return android_1.getAndroidTaskScheduler(); });
        common_2.setTaskChainLauncherCreator(function () { return android_2.getAndroidTaskChainLauncher(); });
        this.wireUpNativeComponents();
        return _super.prototype.init.call(this, appTasks, appTaskGraph, config);
    };
    TaskDispatcher.prototype.isReady = function () {
        android_1.getAndroidTaskScheduler().setup();
        if (this.requiresDisablingBatteryOptimizations()) {
            return Promise.resolve(false);
        }
        return _super.prototype.isReady.call(this);
    };
    TaskDispatcher.prototype.prepare = function () {
        if (this.requiresDisablingBatteryOptimizations()) {
            this.requestToDisableBatteryOptimizations();
        }
        return _super.prototype.prepare.call(this);
    };
    TaskDispatcher.prototype.requiresDisablingBatteryOptimizations = function () {
        if (browser_1.taskGraphBrowser.any(function (task) {
            return task.interval !== 0 &&
                task.interval <= BATTERY_OPTIMIZATIONS_THRESHOLD;
        })) {
            return !power_savings_manager_android_1.getPowerSavingsManager().areDisabled();
        }
        return false;
    };
    TaskDispatcher.prototype.requestToDisableBatteryOptimizations = function () {
        var manager = power_savings_manager_android_1.getPowerSavingsManager();
        if (manager.areDisabled()) {
            return;
        }
        manager.requestDeactivation();
    };
    TaskDispatcher.prototype.wireUpNativeComponents = function () {
        this.wireUpBootReceiver();
        this.wireUpAlarmReceiver();
        this.wireUpAlarmRunnerService();
        this.wireUpWatchdogReceiver();
        this.wireUpTaskChainRunnerService();
    };
    TaskDispatcher.prototype.wireUpBootReceiver = function () {
        es.uji.geotec.taskdispatcher.BootReceiver.setBootReceiverDelegate(new es.uji.geotec.taskdispatcher.BootReceiverDelegate({
            onReceive: function (context, intent) {
                return boot_receiver_android_1.getBootReceiver().onReceive(context, intent);
            },
        }));
    };
    TaskDispatcher.prototype.wireUpAlarmReceiver = function () {
        es.uji.geotec.taskdispatcher.alarms.AlarmReceiver.setAlarmReceiverDelegate(new es.uji.geotec.taskdispatcher.alarms.AlarmReceiverDelegate({
            onReceive: function (context, intent) {
                return receiver_android_1.getAlarmReceiver().onReceive(context, intent);
            },
        }));
    };
    TaskDispatcher.prototype.wireUpAlarmRunnerService = function () {
        es.uji.geotec.taskdispatcher.alarms.AlarmRunnerService.setAlarmRunnerServiceDelegate(new es.uji.geotec.taskdispatcher.alarms.AlarmRunnerServiceDelegate({
            onCreate: function (nativeService) {
                return runner_service_android_1.getAlarmRunnerService().onCreate(nativeService);
            },
            onStartCommand: function (intent, flags, startId) {
                return runner_service_android_1.getAlarmRunnerService().onStartCommand(intent, flags, startId);
            },
            onDestroy: function () { return runner_service_android_1.getAlarmRunnerService().onDestroy(); },
        }));
    };
    TaskDispatcher.prototype.wireUpWatchdogReceiver = function () {
        es.uji.geotec.taskdispatcher.alarms.WatchdogReceiver.setWatchdogReceiverDelegate(new es.uji.geotec.taskdispatcher.alarms.WatchdogReceiverDelegate({
            onReceive: function (context, intent) {
                return receiver_android_2.getWatchDogReceiver().onReceive(context, intent);
            },
        }));
    };
    TaskDispatcher.prototype.wireUpTaskChainRunnerService = function () {
        es.uji.geotec.taskdispatcher.runners.TaskChainRunnerService.setTaskChainRunnerServiceDelegate(new es.uji.geotec.taskdispatcher.runners.TaskChainRunnerServiceDelegate({
            onCreate: function (nativeService) {
                return runner_service_android_2.getTaskChainRunnerService().onCreate(nativeService);
            },
            onStartCommand: function (intent, flags, startId) {
                return runner_service_android_2.getTaskChainRunnerService().onStartCommand(intent, flags, startId);
            },
            onDestroy: function () { return runner_service_android_2.getTaskChainRunnerService().onDestroy(); },
        }));
    };
    return TaskDispatcher;
}(task_dispatcher_common_1.Common));
exports.taskDispatcher = new TaskDispatcher();
//# sourceMappingURL=task-dispatcher.android.js.map